﻿using Microsoft.Office.Interop.Outlook;
using System;
using System.Collections.Generic;

namespace TestProject
{
    /// <summary>
    /// 폴더 컨테이너 관리자
    /// </summary>
    public class FolderContainerManager : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 폴더 삭제시 이벤트 - FolderRemove

        /// <summary>
        /// 폴더 삭제시 이벤트
        /// </summary>
        public event EventHandler FolderRemove;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 세션
        /// </summary>
        private NameSpace session;

        /// <summary>
        /// 폴더 컨테이너 딕셔너리
        /// </summary>
        private Dictionary<string, FolderContainer> folderContainerDictionary = new Dictionary<string, FolderContainer>();

        /// <summary>
        /// 마지막 폴더 삭제 시간
        /// </summary>
        private DateTime lastFolderRemoveTime = DateTime.Now;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - FolderContainerManager(session)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="session">세션</param>
        public FolderContainerManager(NameSpace session)
        {
            this.session = session;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public
        //////////////////////////////////////////////////////////////////////////////// Function

        #region 폴더 컨테이너 딕셔너리 데이터 지우기 - ClearFolderContainerDictionaryData()

        /// <summary>
        /// 폴더 컨테이너 딕셔너리 데이터 지우기
        /// </summary>
        public void ClearFolderContainerDictionaryData()
        {
            foreach(KeyValuePair<string, FolderContainer> keyValuePair in this.folderContainerDictionary)
            {
                string          sourceKey = keyValuePair.Key;
                FolderContainer container = keyValuePair.Value;

                try
                {
                    container.SourceFolder.BeforeFolderMove -= sourceFolder_BeforeFolderMove;
                    container.SourceFolder.BeforeItemMove   -= sourceFolder_BeforeItemMove;
                    container.SourceFolders.FolderAdd       -= sourceFolders_FolderAdd;
                    container.SourceFolders.FolderChange    -= sourceFolders_FolderChange;
                }
                catch
                {
                }

                container.Parent        = null;
                container.SourceKey     = null;
                container.SourceFolder  = null;
                container.SourceFolders = null;

                container.ChildList.Clear();
            }

            this.folderContainerDictionary.Clear();
        }

        #endregion
        #region 폴더 컨테이너 딕셔너리 구축하기 - BuildFolderContainerDictionary()

        /// <summary>
        /// 폴더 컨테이너 딕셔너리 구축하기
        /// </summary>
        public void BuildFolderContainerDictionary()
        {
            foreach(Folder rootfolder in this.session.Folders)
            {
                Folder salesForceFolder = rootfolder.GetFolder(ConstantHelper.SalesForceFolderName);

                if(salesForceFolder == null)
                {
                    continue;
                }

                AddFolderContainer(null, salesForceFolder);
            }
        }

        #endregion

        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            ClearFolderContainerDictionaryData();

            this.session = null;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 소스 폴더 폴더 이동 전 처리하기 - sourceFolder_BeforeFolderMove(targetFolder, cancel)

        /// <summary>
        /// 소스 폴더 폴더 이동 전 처리하기
        /// </summary>
        /// <param name="targetFolder">타겟 폴더</param>
        /// <param name="cancel">취소 여부</param>
        private void sourceFolder_BeforeFolderMove(MAPIFolder targetFolder, ref bool cancel)
        {
            if(targetFolder.Name != "Deleted Items")
            {
                cancel = true;
            }
        }

        #endregion
        #region 소스 폴더 항목 이동 전 처리하기 - sourceFolder_BeforeItemMove(sourceItem, targetFolder, cancel)

        /// <summary>
        /// 소스 폴더 항목 이동 전 처리하기
        /// </summary>
        /// <param name="sourceItem">소스 항목</param>
        /// <param name="targetFolder">타겟 폴더</param>
        /// <param name="cancel">취소 여부</param>
        private void sourceFolder_BeforeItemMove(object sourceItem, MAPIFolder targetFolder, ref bool cancel)
        {
            if(targetFolder.Name != "Deleted Items")
            {
                cancel = true;
            }
        }

        #endregion
        #region 소스 폴더 컬렉션 폴더 추가시 처리하기 - sourceFolders_FolderAdd(folder)

        /// <summary>
        /// 소스 폴더 컬렉션 폴더 추가시 처리하기
        /// </summary>
        /// <param name="folder">폴더</param>
        private void sourceFolders_FolderAdd(MAPIFolder folder)
        {
            try
            {
                ClearFolderContainerDictionaryData();

                BuildFolderContainerDictionary();
            }
            catch(System.Exception)
            {
                return;
            }
        }

        #endregion
        #region 소스 폴더 컬렉션 폴더 변경시 처리하기 - sourceFolders_FolderChange(folder)

        /// <summary>
        /// 소스 폴더 컬렉션 폴더 변경시 처리하기
        /// </summary>
        /// <param name="folder">폴더</param>
        private void sourceFolders_FolderChange(MAPIFolder folder)
        {
            try
            {
                ClearFolderContainerDictionaryData();

                BuildFolderContainerDictionary();
            }
            catch(System.Exception)
            {
                return;
            }
        }

        #endregion
        #region 소스 폴더 컬렉션 폴더 삭제시 처리하기 - sourceFolders_FolderRemove()

        /// <summary>
        /// 소스 폴더 컬렉션 폴더 삭제시 처리하기
        /// </summary>
        private void sourceFolders_FolderRemove()
        {
            TimeSpan timeSpan = DateTime.Now - this.lastFolderRemoveTime;

            if(timeSpan.Milliseconds < 500)
            {
                return;
            }

            FolderRemove?.Invoke(this, EventArgs.Empty);

            this.lastFolderRemoveTime = DateTime.Now;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 폴더 컨테이너 추가하기 - AddFolderContainer(parentContainer, folder)

        /// <summary>
        /// 폴더 컨테이너 추가하기
        /// </summary>
        /// <param name="parentContainer">부모 컨테이너</param>
        /// <param name="folder">폴더</param>
        private void AddFolderContainer(FolderContainer parentContainer, Folder folder)
        {
            string sourceKey = folder.FullFolderPath.Substring(2);

            string[] sourceKeyItemArray = sourceKey.Split('\\');

            if(sourceKeyItemArray.Length < 2 || sourceKeyItemArray[1] != ConstantHelper.SalesForceFolderName)
            {
                return;
            }

            FolderContainer container = new FolderContainer();

            container.Parent        = parentContainer;
            container.SourceKey     = sourceKey;
            container.SourceFolder  = folder;
            container.SourceFolders = folder.Folders;

            container.SourceFolder.BeforeFolderMove -= sourceFolder_BeforeFolderMove;
            container.SourceFolder.BeforeItemMove   -= sourceFolder_BeforeItemMove;
            container.SourceFolders.FolderAdd       -= sourceFolders_FolderAdd;
            container.SourceFolders.FolderChange    -= sourceFolders_FolderChange;
            container.SourceFolders.FolderRemove    -= sourceFolders_FolderRemove;

            container.SourceFolder.BeforeFolderMove += sourceFolder_BeforeFolderMove;
            container.SourceFolder.BeforeItemMove   += sourceFolder_BeforeItemMove;
            container.SourceFolders.FolderAdd       += sourceFolders_FolderAdd;
            container.SourceFolders.FolderChange    += sourceFolders_FolderChange;
            container.SourceFolders.FolderRemove    += sourceFolders_FolderRemove;

            if(parentContainer != null)
            {
                parentContainer.ChildList.Add(container);
            }

            this.folderContainerDictionary.Add(container.SourceKey, container);

            //Debug.WriteLine(container.SourceKey);

            foreach(Folder childFolder in folder.Folders)
            {
                AddFolderContainer(container, childFolder);
            }
        }

        #endregion
    }
}